/*
   Copyright (C) 1994, 1996 Free Software Foundation

This file is part of the GNU Hurd.

The GNU Hurd is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

The GNU Hurd is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GNU Hurd; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Written by Michael I. Bushnell.  */

/* Hmm. */

#define dithkfth_TH_file_chauthor diskfs_S_file_chauthor
#define fthhelp_ithowner fshelp_isowner
#define dithkfth_validate_author_change diskfs_validate_author_change
#define dn_thtat dn_stat
#define tht_author st_author
#define dn_thet_theetime dn_set_ctime
#define fth_TH_dot_h "fs_S.h"
#define uther user
